clear all
close all
clc

load 'TP6.mat'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Quantile Estimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Historical quantile
Hist_q=quantile(StocksWeeklyReturns(:,1),0.01)%matlab function quantile
quantile(StocksWeeklyReturns(:,1),0)
% replication of the matlab function quantile
n_r=length(StocksWeeklyReturns(:,1));
[r_sorted,index_r_sorted]=sort(StocksWeeklyReturns(:,1));
interp1([100*(0.5:(n_r-0.5))./n_r ]',r_sorted,1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VaR estimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1-2.
%portfolio weight vector a, following the notation in the lecture notes
%a=[1 0 0 0 0 0 0 0 0 0]'
a=[.1 .1 .1 .1 .1 .1 .1 .1 .1 .1]'; %change the portfolio weights and experiment!
%a=[.23 .05 .13 .08 .1 .06 .28 .01 .04 .02]'; %another fancy portfolio

fprintf('------- Question 1 -------\n')
fprintf('------- Mean -------\n')
mu=mean(StocksWeeklyReturns)'       %empirical mean
fprintf('------- Var/Covar Matrix -------\n')
SIGMA=cov(StocksWeeklyReturns)      %empirical var-covar matrix
fprintf('------- Skewness -------\n')
skew=skewness(StocksWeeklyReturns)  %empirical skewness
fprintf('------- Kurtosis -------\n')
kurt=kurtosis(StocksWeeklyReturns)  %empirical kurtosis

fprintf('------- Question 2 -------\n')
z_95=norminv(.95,0,1); %Inverse normal cumulative distribution function giving the 95% quantile
GaussianVaR=-(a'*mu-(a'*SIGMA*a)^(1/2)*z_95)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3.

z=-StocksWeeklyReturns*a; %This is the negative of the returns weighted by their portfolio weight (a)

opt=optimset('Display','iter'); %Optimization parameters
guess=GaussianVaR; %guess for the numerical minimization procedure.
NonParamVaR=fminsearch(@obj_fct, guess, opt, z)

%We see that the Gaussian Hypothesis seems to underestimate the true VAR!!! That's
%a major reason why it's so problematic to assume Gaussian returns!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sensitivities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

SensitivitiesGaussianVaR=-mu+SIGMA*a/(a'*SIGMA*a)^(1/2)*z_95 %easy direct formula for Gaussian Hypothesis

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2.

%Here we implement formula on page AX-77 in Lecture Notes
%Vectorialized code:
T=length(z);
h=std(z)*T^(-.2);
C=exp(-.5*((-StocksWeeklyReturns*a-NonParamVaR)/h).^2);
A=kron(ones(1,10),C);
B=-StocksWeeklyReturns.*A;

SensitivitiesNonParamVaR=sum(B)'/sum(C)

% Another way to do the same thing (but with less speed) is with a loop:
% num=zeros(10,1);
% denom=0;
% for i=1:T
%     num=num-StocksWeeklyReturns(i,:)' * exp(-.5*((-a'*StocksWeeklyReturns(i,:)'-NonParamVaR)/h)^2);
%     denom=denom+exp(-.5*((-a'*StocksWeeklyReturns(i,:)'-NonParamVaR)/h)^2);
% end
% SensitivitiesNonParamVaR_loop=num/denom

% In any case, you can see substantial differences, mostly due to
% departure from non-normality of returns (significant excess kurtosis and
% skewness)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Comparison with mean-variance optimization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

% from your asset allocation class:

mu_p=.05; %this is the target return in the Problem, i.e., 5%
opt=optimset('LargeScale','off');
guess=ones(10,1)/10;

ww=fmincon('multivar', guess,[], [], [mu'; ones(1, length(mu))],[mu_p;1],[],[],[],opt,SIGMA) 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2.

aa=fmincon(@(a) GaussianVaR_fct(a,mu,SIGMA,0.05),guess,[],[],[mu'; ones(1,length(mu))],[mu_p; 1],[],[],[],opt)
 [ww aa]
% note that we have to use fmincon in order to control the return of the
% portfolio.

VaR_mean_variance=GaussianVaR_fct(ww,mu,SIGMA,0.05)
VaR_mean_VaR=GaussianVaR_fct(aa,mu,SIGMA,0.05)

% As you can see, mean-variance efficiency does not imply mean-VaR
% efficiency.
