function kernel_pdf=gaussiankernel(obs,nb_points,bandwidth)
% kernel_pdf=gaussiankernel(obs,nb_points,bandwidth)
% 
% la fonction gaussiankernel estime la densit de la distribution
% sous-jacente de l'chantillon obs par la mthode non-paramtrique du
% noyaux gaussien.
%
% Input:
% - obs est un vecteur contenant les observations
% - nb_points est le nombre de points pour lesquels la ensit est estime
% (voir remarque)
% - bandwidth est un argument optionnel
%
% Output:
% - kernel_pdf est une matrice (2,nbpoint) dont la premire ligne contient
% les points d'estimation et la deuxime ligne les valeurs de la densit.
%
% Remarque:
% la densit est estime entre le min et le max de l'chantillon
% d'observations, pour nb_points points equidistants.
%

% Remarque: tous les commentaires sur la fonction ci-dessus seront
% affichs par Matlab si l'on tape help gaussiankernel dans la fenetre de
% commande.

nb_obs=length(obs);

% les calculs suivants ncessitent que obs soit un vecteur colonne.
obs=reshape(obs,nb_obs,1);

% Le 3eme input argument de la fonction gaussiankernel est optionnel. Si
% aucune valeur n'est transmise pour bandwith, le programme utilise la
% "rule of thumb"
if nargin==2
    bandwidth=(nb_obs^(-0.2))*(var(obs)^0.5);
end

% cration de la grille de points
max_obs=max(obs);
min_obs=min(obs);
step=(max_obs-min_obs)/(nb_points-1);
kernel_pdf=[min_obs:step:max_obs];

% on calcule tous les points en une seule passe en utilisant les fonctions
% mean et le produit de kronecker.
% c'est l'criture la plus compacte mais SURTOUT la plus rapide car elle
% utilise au maximum les fonctions prprogrammes de Matlab.
% Matlab tant un language interprt (par oppostition  compil), il faut
% viter l'usage des boucles et "vectoriser" le code au maximum pour
% obtenir une bonne performance.
% Consultez les aides sur les diffrentes fonctions utilises pour plus de
% dtails.
kernel_pdf=[kernel_pdf; mean(exp(-0.5*((kron(obs,ones(1,nb_points))-kron(ones(nb_obs,1),kernel_pdf))/bandwidth).^2),1)/bandwidth/((2*pi)^0.5)];

